//Xilinx SDK includes
#include "xparameters.h"
#include "stdio.h"
#include "xsysmon.h"
#include "xtmrctr.h"
#include "xio.h"

//WARP includes
#include "w3_userio.h"
#include "w3_ad_controller.h"
#include "w3_clock_controller.h"
#include "w3_iic_eeprom.h"
#include "radio_controller.h"

//Define standard macros for pcore base addresses and device IDs
// XPAR_ names will change with instance names in hardware
#define USERIO_BASEADDR		XPAR_W3_USERIO_0_BASEADDR
#define RC_BASEADDR 		XPAR_RADIO_CONTROLLER_0_BASEADDR
#define AD_BASEADDR 		XPAR_W3_AD_CONTROLLER_0_BASEADDR
#define CLK_BASEADDR 		XPAR_W3_CLOCK_CONTROLLER_0_BASEADDR
#define EEPROM_BASEADDR 	XPAR_W3_IIC_EEPROM_ONBOARD_BASEADDR
#define DRAM_BASEADDR 		XPAR_DDR3_2GB_SODIMM_MPMC_BASEADDR
#define TIMER_FREQ			XPAR_XPS_TIMER_0_CLOCK_FREQ_HZ
#define TMRCTR_DEVICE_ID	XPAR_TMRCTR_0_DEVICE_ID
#define CAPTURE_PERIOD		XPAR_PRNG_USERIOSRC_PLBW_0_MEMMAP_CAPTUREPERIOD
#define CAPTURED_OUTPUT		XPAR_PRNG_USERIOSRC_PLBW_0_MEMMAP_CAPTUREDOUTPUT


//Global variable definitions
XTmrCtr TimerCounter; /* The instance of the Tmrctr Device */

int w3_node_init();
void userio_example();
void usleep(u32 duration);

int main() {
	int status;

	xil_printf("                \f");
	xil_printf("WARP v3 Template Project - Lite\n");

	status = w3_node_init();
	if(status != 0) {
		xil_printf("Error in w3_node_init()! Exiting\n");
		return -1;
	}

	xil_printf("Board serial number: W3-a-%05d\n\n", w3_eeprom_readSerialNum(EEPROM_BASEADDR));

	xil_printf("Running User I/O Example\n");
	userio_example();

	return 0;
}

void userio_example() {
	u16 capturedValue;
	u8 i,allowPrint;

	/*
	 * We use allowPrint as a simple software "latch" to avoid unnecessary prints to the UART.
	 * When the user presses and holds the "up" pushbutton on the board, this code will print
	 * the binary string of the output of the custom core's LFSR. When the user releases the
	 * pushbutton, this code will print a series of backspaces to delete the previous print.
	 */
	allowPrint = 1;

	/*
	 * The User I/O core can be controlled from software registers or from its hardware port
	 * inputs. In this exercise, we are driving the core from its ports.
	 * The userio_set_ctrlSrc_hw() macro allows configures the core to ignore its register
	 * inputs and instead use its hardware ports.
	 */
	userio_set_ctrlSrc_hw(USERIO_BASEADDR, (W3_USERIO_CTRLSRC_LEDS|W3_USERIO_CTRLSRC_HEXDISPS));

	/*
	 * The CAPTURE_PERIOD definition simply abstracts away the address of the "capturePeriod"
	 * register that was created in the underlying custom pcore. XIo_Out32 is a simple macro
	 * that Xilinx provides that lets us write values to that memory address. In this function,
	 * we write a value that will ensure that the USER I/O LEDs are updated at 10 times per second.
	 * Reduce this value to increase the speed of the updates and increase it to slow it down further.
	 */
	XIo_Out32(CAPTURE_PERIOD,8000000-1);

	xil_printf("\t            Left Hex  -------------     -------------  Right Hex\n");
	xil_printf("\t            Red LEDs  -------                 -------  Green LEDs\n");
	xil_printf("\tPRNG Captured Value:                                  ");

	/*
	 * This while loop will run forever. There is no exit condition to break out of this loop.
	 * This loop will continually read from the User I/O core to determine if any pushbuttons
	 * are currently being pressed. If so, it triggers a small loop to print out the output
	 * of the custom pcore's capturedOutput register. This inner loop is simply to display
	 * the output as a binary string.
	 */
	while(1){
		if((userio_read_inputs(USERIO_BASEADDR) & W3_USERIO_PB_U)){
			if(allowPrint){
				capturedValue = XIo_In32(CAPTURED_OUTPUT);
				xil_printf("\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b");
				for (i=0;i<16;i++){
					xil_printf("%d ", (capturedValue>>(15-i))&1);
				}
				allowPrint = 0;
			}
		} else if(!allowPrint) {
			xil_printf("\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b");
			xil_printf("                                ");
			allowPrint = 1;
		}
	}
	xil_printf("\n");
	return;

}

int w3_node_init() {

	int status;
	XTmrCtr *TmrCtrInstancePtr = &TimerCounter;
	int ret = XST_SUCCESS;

	microblaze_enable_exceptions();

	//Initialize the AD9512 clock buffers (RF reference and sampling clocks)
	status = clk_init(CLK_BASEADDR, 2);
	if(status != XST_SUCCESS) {
		xil_printf("w3_node_init: Error in clk_init (%d)\n", status);
		ret = XST_FAILURE;
	}

	//Initialize the AD9963 ADCs/DACs
	ad_init(AD_BASEADDR, (RFA_AD_CS | RFB_AD_CS), 2);
	if(status != XST_SUCCESS) {
		xil_printf("w3_node_init: Error in ad_init (%d)\n", status);
		ret = XST_FAILURE;
	}

	//Initialize the radio_controller core and MAX2829 transceivers
	status = radio_controller_init(RC_BASEADDR, (RC_RFA | RC_RFB), 1, 1);
	if(status != XST_SUCCESS) {
		xil_printf("w3_node_init: Error in radioController_initialize (%d)\n", status);
		ret = XST_FAILURE;
	}

	//Initialize the EEPROM read/write core
	iic_eeprom_init(EEPROM_BASEADDR, 0x64);
	if(status != XST_SUCCESS) {
		xil_printf("w3_node_init: Error in IIC_EEPROM_init (%d)\n", status);
		ret = XST_FAILURE;
	}

	/*
	 * Initialize the timer counter so that it's ready to use,
	 * specify the device ID that is generated in xparameters.h
	 */
	status = XTmrCtr_Initialize(TmrCtrInstancePtr, TMRCTR_DEVICE_ID);
	if (status != XST_SUCCESS) {
		xil_printf("w3_node_init: Error in XtmrCtr_Initialize (%d)\n", status);
		ret = XST_FAILURE;
	}

	/*
	 * Perform a self-test to ensure that the hardware was built
	 * correctly, use the 1st timer in the device (0)
	 */
	status = XTmrCtr_SelfTest(TmrCtrInstancePtr, 0);
	if (status != XST_SUCCESS) {
		xil_printf("w3_node_init: Error in XTmrCtr_SelfTest (%d)\n", status);
		ret = XST_FAILURE;
	}

	// Set timer 0 to into a "count down" mode
	XTmrCtr_SetOptions(TmrCtrInstancePtr, 0, (XTC_DOWN_COUNT_OPTION));

	return ret;
}

void usleep(u32 duration){
	XTmrCtr *TmrCtrInstancePtr = &TimerCounter;
	XTmrCtr_SetResetValue(TmrCtrInstancePtr,0,duration*(TIMER_FREQ/1000000));

	XTmrCtr_Start(TmrCtrInstancePtr,0);

	volatile u8 isExpired = 0;
	while(isExpired!=1){
		isExpired = XTmrCtr_IsExpired(TmrCtrInstancePtr,0);
	}
	XTmrCtr_Reset(TmrCtrInstancePtr,0);
	return;
}

